--------------------------------------------------------------------
-- Enumeration types -----------------------------------------------

CREATE TYPE detection_event_action_enum AS ENUM ('LOG', 'BLOCK');

CREATE TYPE compute_type_enum AS ENUM ('IPV4', 'IPV6', 'VMUUID', 'BMUUID');

CREATE TYPE detection_type_enum AS ENUM ('IDS', 'NETWORK_ANOMALY', 'FILE', 'FILE_TRANSFER');

-- add IP, Domain, URL for reputation hits?
-- add yara rule?
CREATE TYPE indicator_of_compromise_type_enum AS ENUM ('FILE_SHA256');

CREATE TYPE transport_protocol_enum as ENUM ('TCP', 'UDP', 'ESP', 'GRE', 'SCTP');

CREATE TYPE transport_node_type_enum as ENUM ('HOST_NODE', 'EDGE_NODE', 'STANDALONE_SENSOR');

CREATE TYPE verification_outcome_enum AS ENUM ('SUCCESSFUL','UNKNOWN','FAILED','BLOCKED');

CREATE TYPE siem_message_format_enum AS ENUM ('SPLUNK_JSON', 'VRLI_JSON', 'DEFAULT');

--------------------------------------------------------------------
-- Trigger functions -----------------------------------------------

CREATE OR REPLACE FUNCTION update_last_modified_column()
RETURNS TRIGGER AS $$
BEGIN
    NEW.last_modified = now();
    RETURN NEW;
END;
$$ language 'plpgsql';

CREATE OR REPLACE FUNCTION update_revision_column()
RETURNS TRIGGER AS $$
BEGIN
    NEW.revision = OLD.revision + 1;
    RETURN NEW;
END;
$$ language 'plpgsql';

--------------------------------------------------------------------
-- raw_detection_event ---------------------------------------------

CREATE TABLE raw_detection_event(
    detection_event_id BIGSERIAL NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    detection_event_uuid UUID NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    action detection_event_action_enum NOT NULL,
    affected_compute_type compute_type_enum NOT NULL,
    affected_compute_uuid UUID NOT NULL,
    affected_compute_ip INET NULL,
    affected_compute_is_private BOOL NOT NULL,
    -- optional because site/deployment uuid may be missing in basic single-cluster scenario
    site_uuid UUID NULL DEFAULT NULL,
    client_affected BOOL NULL DEFAULT NULL,
    target_affected BOOL NULL DEFAULT NULL,
    detection_type detection_type_enum NOT NULL,
    external_detection_uuid UUID NULL DEFAULT NULL,
    detector_uuid UUID NULL DEFAULT NULL,
    indicator_of_compromise_type indicator_of_compromise_type_enum NULL DEFAULT NULL,
    -- need a relatively long size for URLs and such
    -- (though could store in separate lookup table if a type is just too long?
    indicator_of_compromise VARCHAR(512),
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    last_modified TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    PRIMARY KEY (detection_event_id, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE raw_detection_event_partition_month_id_0 PARTITION OF raw_detection_event FOR VALUES IN (0);
CREATE TABLE raw_detection_event_partition_month_id_1 PARTITION OF raw_detection_event FOR VALUES IN (1);
CREATE TABLE raw_detection_event_partition_month_id_2 PARTITION OF raw_detection_event FOR VALUES IN (2);
CREATE TABLE raw_detection_event_partition_month_id_3 PARTITION OF raw_detection_event FOR VALUES IN (3);
CREATE TABLE raw_detection_event_partition_month_id_4 PARTITION OF raw_detection_event FOR VALUES IN (4);
CREATE TABLE raw_detection_event_partition_month_id_5 PARTITION OF raw_detection_event FOR VALUES IN (5);
CREATE TABLE raw_detection_event_partition_month_id_6 PARTITION OF raw_detection_event FOR VALUES IN (6);
CREATE TABLE raw_detection_event_partition_month_id_7 PARTITION OF raw_detection_event FOR VALUES IN (7);
CREATE TABLE raw_detection_event_partition_month_id_8 PARTITION OF raw_detection_event FOR VALUES IN (8);
CREATE TABLE raw_detection_event_partition_month_id_9 PARTITION OF raw_detection_event FOR VALUES IN (9);
CREATE TABLE raw_detection_event_partition_month_id_10 PARTITION OF raw_detection_event FOR VALUES IN (10);
CREATE TABLE raw_detection_event_partition_month_id_11 PARTITION OF raw_detection_event FOR VALUES IN (11);
CREATE TABLE raw_detection_event_partition_month_id_12 PARTITION OF raw_detection_event FOR VALUES IN (12);
CREATE TABLE raw_detection_event_partition_month_id_13 PARTITION OF raw_detection_event FOR VALUES IN (13);
CREATE TABLE raw_detection_event_partition_month_id_14 PARTITION OF raw_detection_event FOR VALUES IN (14);
CREATE TABLE raw_detection_event_partition_month_id_15 PARTITION OF raw_detection_event FOR VALUES IN (15);
CREATE TABLE raw_detection_event_partition_month_id_16 PARTITION OF raw_detection_event FOR VALUES IN (16);
CREATE TABLE raw_detection_event_partition_month_id_17 PARTITION OF raw_detection_event FOR VALUES IN (17);
CREATE TABLE raw_detection_event_partition_month_id_18 PARTITION OF raw_detection_event FOR VALUES IN (18);
CREATE TABLE raw_detection_event_partition_month_id_19 PARTITION OF raw_detection_event FOR VALUES IN (19);
CREATE TABLE raw_detection_event_partition_month_id_20 PARTITION OF raw_detection_event FOR VALUES IN (20);
CREATE TABLE raw_detection_event_partition_month_id_21 PARTITION OF raw_detection_event FOR VALUES IN (21);
CREATE TABLE raw_detection_event_partition_month_id_22 PARTITION OF raw_detection_event FOR VALUES IN (22);
CREATE TABLE raw_detection_event_partition_month_id_23 PARTITION OF raw_detection_event FOR VALUES IN (23);

CREATE UNIQUE INDEX raw_detection_event_unique_uuid ON raw_detection_event (detection_event_uuid, partition_month_id);

CREATE INDEX raw_detection_event_idx_start ON raw_detection_event (start_time);

-- index on uuid before type, so in a pinch we can find a uuid without knowing type
CREATE INDEX raw_detection_event_idx_affected_compute ON raw_detection_event (
    affected_compute_uuid,
    affected_compute_type,
    start_time
);

CREATE INDEX raw_detection_event_idx_affected_ip ON raw_detection_event (
    affected_compute_ip,
    start_time
);

CREATE INDEX raw_detection_event_idx_external_detection ON raw_detection_event (external_detection_uuid);

CREATE INDEX raw_detection_event_idx_detector ON raw_detection_event (
    detector_uuid,
    start_time
);

CREATE INDEX raw_detection_event_idx_ioc ON raw_detection_event (
    indicator_of_compromise_type,
    indicator_of_compromise,
    start_time
);

CREATE TRIGGER update_raw_detection_event_modtime
    BEFORE UPDATE ON raw_detection_event
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_last_modified_column();

--------------------------------------------------------------------
-- detection_event -------------------------------------------------

CREATE TABLE detection_event(
    detection_event_id BIGINT NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    detection_event_uuid UUID NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    action detection_event_action_enum NOT NULL,
    affected_compute_type compute_type_enum NOT NULL,
    affected_compute_uuid UUID NOT NULL,
    affected_compute_ip INET NULL,
    affected_compute_is_private BOOL NOT NULL,
    site_uuid UUID NULL DEFAULT NULL,
    client_affected BOOL NULL DEFAULT NULL,
    target_affected BOOL NULL DEFAULT NULL,
    detection_type detection_type_enum NOT NULL,
    external_detection_uuid UUID NULL DEFAULT NULL,
    detector_uuid UUID NULL DEFAULT NULL,
    indicator_of_compromise_type indicator_of_compromise_type_enum NULL DEFAULT NULL,
    indicator_of_compromise VARCHAR(512),
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    last_modified TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    impact SMALLINT NOT NULL,
    confidence SMALLINT NOT NULL,
    severity SMALLINT NOT NULL,
    experimental BOOL NOT NULL,
    mitre_tactic_uuid UUID NOT NULL,
    mitre_technique_uuid UUID NULL,
    threat_uuid UUID NOT NULL,
    verification_outcome verification_outcome_enum NOT NULL,
    verifier_uuid UUID NULL,
    PRIMARY KEY (detection_event_id, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_partition_month_id_0 PARTITION OF detection_event FOR VALUES IN (0);
CREATE TABLE detection_event_partition_month_id_1 PARTITION OF detection_event FOR VALUES IN (1);
CREATE TABLE detection_event_partition_month_id_2 PARTITION OF detection_event FOR VALUES IN (2);
CREATE TABLE detection_event_partition_month_id_3 PARTITION OF detection_event FOR VALUES IN (3);
CREATE TABLE detection_event_partition_month_id_4 PARTITION OF detection_event FOR VALUES IN (4);
CREATE TABLE detection_event_partition_month_id_5 PARTITION OF detection_event FOR VALUES IN (5);
CREATE TABLE detection_event_partition_month_id_6 PARTITION OF detection_event FOR VALUES IN (6);
CREATE TABLE detection_event_partition_month_id_7 PARTITION OF detection_event FOR VALUES IN (7);
CREATE TABLE detection_event_partition_month_id_8 PARTITION OF detection_event FOR VALUES IN (8);
CREATE TABLE detection_event_partition_month_id_9 PARTITION OF detection_event FOR VALUES IN (9);
CREATE TABLE detection_event_partition_month_id_10 PARTITION OF detection_event FOR VALUES IN (10);
CREATE TABLE detection_event_partition_month_id_11 PARTITION OF detection_event FOR VALUES IN (11);
CREATE TABLE detection_event_partition_month_id_12 PARTITION OF detection_event FOR VALUES IN (12);
CREATE TABLE detection_event_partition_month_id_13 PARTITION OF detection_event FOR VALUES IN (13);
CREATE TABLE detection_event_partition_month_id_14 PARTITION OF detection_event FOR VALUES IN (14);
CREATE TABLE detection_event_partition_month_id_15 PARTITION OF detection_event FOR VALUES IN (15);
CREATE TABLE detection_event_partition_month_id_16 PARTITION OF detection_event FOR VALUES IN (16);
CREATE TABLE detection_event_partition_month_id_17 PARTITION OF detection_event FOR VALUES IN (17);
CREATE TABLE detection_event_partition_month_id_18 PARTITION OF detection_event FOR VALUES IN (18);
CREATE TABLE detection_event_partition_month_id_19 PARTITION OF detection_event FOR VALUES IN (19);
CREATE TABLE detection_event_partition_month_id_20 PARTITION OF detection_event FOR VALUES IN (20);
CREATE TABLE detection_event_partition_month_id_21 PARTITION OF detection_event FOR VALUES IN (21);
CREATE TABLE detection_event_partition_month_id_22 PARTITION OF detection_event FOR VALUES IN (22);
CREATE TABLE detection_event_partition_month_id_23 PARTITION OF detection_event FOR VALUES IN (23);

CREATE UNIQUE INDEX detection_event_unique_uuid ON detection_event (detection_event_uuid, partition_month_id);

CREATE INDEX detection_event_idx_start ON detection_event (start_time);

-- index on uuid before type, so in a pinch we can find a uuid without knowing type
CREATE INDEX detection_event_idx_affected_compute ON detection_event (
    affected_compute_uuid,
    affected_compute_type,
    start_time
);

CREATE INDEX detection_event_idx_affected_ip ON detection_event (
    affected_compute_ip,
    start_time
);

CREATE INDEX detection_event_idx_external_detection ON detection_event (external_detection_uuid);

CREATE INDEX detection_event_idx_detector ON detection_event (
    detector_uuid,
    start_time
);

CREATE INDEX detection_event_idx_ioc ON detection_event (
    indicator_of_compromise_type,
    indicator_of_compromise,
    start_time
);

CREATE INDEX detection_event_idx_mitre_attack ON detection_event (
    mitre_tactic_uuid,
    mitre_technique_uuid,
    start_time
);

CREATE INDEX detection_event_idx_mitre_technique ON detection_event (
    mitre_technique_uuid,
    start_time
);

CREATE INDEX detection_event_idx_threat ON detection_event (
    threat_uuid,
    start_time
);

CREATE INDEX detection_event_idx_verifier ON detection_event (
    verifier_uuid,
    start_time
);

CREATE TRIGGER update_detection_event_modtime
    BEFORE UPDATE ON detection_event
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_last_modified_column();

--------------------------------------------------------------------
-- detection_event_label -------------------------------------------

CREATE TABLE detection_event_label(
    detection_event_label_uuid UUID NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    start_time TIMESTAMP NOT NULL,
    detection_event_uuid UUID NOT NULL,
    label_key VARCHAR(64),
    label_value VARCHAR(255),
    PRIMARY KEY (detection_event_label_uuid, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_label_partition_month_id_0 PARTITION OF detection_event_label FOR VALUES IN (0);
CREATE TABLE detection_event_label_partition_month_id_1 PARTITION OF detection_event_label FOR VALUES IN (1);
CREATE TABLE detection_event_label_partition_month_id_2 PARTITION OF detection_event_label FOR VALUES IN (2);
CREATE TABLE detection_event_label_partition_month_id_3 PARTITION OF detection_event_label FOR VALUES IN (3);
CREATE TABLE detection_event_label_partition_month_id_4 PARTITION OF detection_event_label FOR VALUES IN (4);
CREATE TABLE detection_event_label_partition_month_id_5 PARTITION OF detection_event_label FOR VALUES IN (5);
CREATE TABLE detection_event_label_partition_month_id_6 PARTITION OF detection_event_label FOR VALUES IN (6);
CREATE TABLE detection_event_label_partition_month_id_7 PARTITION OF detection_event_label FOR VALUES IN (7);
CREATE TABLE detection_event_label_partition_month_id_8 PARTITION OF detection_event_label FOR VALUES IN (8);
CREATE TABLE detection_event_label_partition_month_id_9 PARTITION OF detection_event_label FOR VALUES IN (9);
CREATE TABLE detection_event_label_partition_month_id_10 PARTITION OF detection_event_label FOR VALUES IN (10);
CREATE TABLE detection_event_label_partition_month_id_11 PARTITION OF detection_event_label FOR VALUES IN (11);
CREATE TABLE detection_event_label_partition_month_id_12 PARTITION OF detection_event_label FOR VALUES IN (12);
CREATE TABLE detection_event_label_partition_month_id_13 PARTITION OF detection_event_label FOR VALUES IN (13);
CREATE TABLE detection_event_label_partition_month_id_14 PARTITION OF detection_event_label FOR VALUES IN (14);
CREATE TABLE detection_event_label_partition_month_id_15 PARTITION OF detection_event_label FOR VALUES IN (15);
CREATE TABLE detection_event_label_partition_month_id_16 PARTITION OF detection_event_label FOR VALUES IN (16);
CREATE TABLE detection_event_label_partition_month_id_17 PARTITION OF detection_event_label FOR VALUES IN (17);
CREATE TABLE detection_event_label_partition_month_id_18 PARTITION OF detection_event_label FOR VALUES IN (18);
CREATE TABLE detection_event_label_partition_month_id_19 PARTITION OF detection_event_label FOR VALUES IN (19);
CREATE TABLE detection_event_label_partition_month_id_20 PARTITION OF detection_event_label FOR VALUES IN (20);
CREATE TABLE detection_event_label_partition_month_id_21 PARTITION OF detection_event_label FOR VALUES IN (21);
CREATE TABLE detection_event_label_partition_month_id_22 PARTITION OF detection_event_label FOR VALUES IN (22);
CREATE TABLE detection_event_label_partition_month_id_23 PARTITION OF detection_event_label FOR VALUES IN (23);

CREATE INDEX detection_event_label_event_uuid ON detection_event_label (detection_event_uuid);

--------------------------------------------------------------------
-- detection_event_observable --------------------------------------

CREATE TABLE detection_event_observable(
    detection_event_observable_uuid UUID NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    detection_event_uuid UUID NOT NULL,
    observable_key VARCHAR(64) NOT NULL,
    observable_value JSONB NOT NULL,
    observable_value_hash VARCHAR(32) NOT NULL,
    PRIMARY KEY (detection_event_observable_uuid, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_observable_partition_month_id_0 PARTITION OF detection_event_observable FOR VALUES IN (0);
CREATE TABLE detection_event_observable_partition_month_id_1 PARTITION OF detection_event_observable FOR VALUES IN (1);
CREATE TABLE detection_event_observable_partition_month_id_2 PARTITION OF detection_event_observable FOR VALUES IN (2);
CREATE TABLE detection_event_observable_partition_month_id_3 PARTITION OF detection_event_observable FOR VALUES IN (3);
CREATE TABLE detection_event_observable_partition_month_id_4 PARTITION OF detection_event_observable FOR VALUES IN (4);
CREATE TABLE detection_event_observable_partition_month_id_5 PARTITION OF detection_event_observable FOR VALUES IN (5);
CREATE TABLE detection_event_observable_partition_month_id_6 PARTITION OF detection_event_observable FOR VALUES IN (6);
CREATE TABLE detection_event_observable_partition_month_id_7 PARTITION OF detection_event_observable FOR VALUES IN (7);
CREATE TABLE detection_event_observable_partition_month_id_8 PARTITION OF detection_event_observable FOR VALUES IN (8);
CREATE TABLE detection_event_observable_partition_month_id_9 PARTITION OF detection_event_observable FOR VALUES IN (9);
CREATE TABLE detection_event_observable_partition_month_id_10 PARTITION OF detection_event_observable FOR VALUES IN (10);
CREATE TABLE detection_event_observable_partition_month_id_11 PARTITION OF detection_event_observable FOR VALUES IN (11);
CREATE TABLE detection_event_observable_partition_month_id_12 PARTITION OF detection_event_observable FOR VALUES IN (12);
CREATE TABLE detection_event_observable_partition_month_id_13 PARTITION OF detection_event_observable FOR VALUES IN (13);
CREATE TABLE detection_event_observable_partition_month_id_14 PARTITION OF detection_event_observable FOR VALUES IN (14);
CREATE TABLE detection_event_observable_partition_month_id_15 PARTITION OF detection_event_observable FOR VALUES IN (15);
CREATE TABLE detection_event_observable_partition_month_id_16 PARTITION OF detection_event_observable FOR VALUES IN (16);
CREATE TABLE detection_event_observable_partition_month_id_17 PARTITION OF detection_event_observable FOR VALUES IN (17);
CREATE TABLE detection_event_observable_partition_month_id_18 PARTITION OF detection_event_observable FOR VALUES IN (18);
CREATE TABLE detection_event_observable_partition_month_id_19 PARTITION OF detection_event_observable FOR VALUES IN (19);
CREATE TABLE detection_event_observable_partition_month_id_20 PARTITION OF detection_event_observable FOR VALUES IN (20);
CREATE TABLE detection_event_observable_partition_month_id_21 PARTITION OF detection_event_observable FOR VALUES IN (21);
CREATE TABLE detection_event_observable_partition_month_id_22 PARTITION OF detection_event_observable FOR VALUES IN (22);
CREATE TABLE detection_event_observable_partition_month_id_23 PARTITION OF detection_event_observable FOR VALUES IN (23);

CREATE UNIQUE INDEX detection_event_observable_unique ON detection_event_observable (
    detection_event_uuid,
    observable_key,
    observable_value_hash,
    partition_month_id
);

CREATE INDEX detection_event_observable_event_uuid ON detection_event_observable (detection_event_uuid);

CREATE INDEX detection_event_observable_key ON detection_event_observable (observable_key, start_time);

--------------------------------------------------------------------
-- detection_event_source ------------------------------------------

CREATE TABLE detection_event_source(
    detection_event_source_uuid UUID NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    detection_event_uuid UUID NOT NULL,
    src_compute_type compute_type_enum NOT NULL,
    src_compute_uuid UUID NOT NULL,
    src_compute_ip INET NULL,
    src_compute_fqdn VARCHAR(255) NULL,
    src_is_private BOOL NOT NULL,
    src_site_uuid UUID NULL,
    PRIMARY KEY (detection_event_source_uuid, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_source_partition_month_id_0 PARTITION OF detection_event_source FOR VALUES IN (0);
CREATE TABLE detection_event_source_partition_month_id_1 PARTITION OF detection_event_source FOR VALUES IN (1);
CREATE TABLE detection_event_source_partition_month_id_2 PARTITION OF detection_event_source FOR VALUES IN (2);
CREATE TABLE detection_event_source_partition_month_id_3 PARTITION OF detection_event_source FOR VALUES IN (3);
CREATE TABLE detection_event_source_partition_month_id_4 PARTITION OF detection_event_source FOR VALUES IN (4);
CREATE TABLE detection_event_source_partition_month_id_5 PARTITION OF detection_event_source FOR VALUES IN (5);
CREATE TABLE detection_event_source_partition_month_id_6 PARTITION OF detection_event_source FOR VALUES IN (6);
CREATE TABLE detection_event_source_partition_month_id_7 PARTITION OF detection_event_source FOR VALUES IN (7);
CREATE TABLE detection_event_source_partition_month_id_8 PARTITION OF detection_event_source FOR VALUES IN (8);
CREATE TABLE detection_event_source_partition_month_id_9 PARTITION OF detection_event_source FOR VALUES IN (9);
CREATE TABLE detection_event_source_partition_month_id_10 PARTITION OF detection_event_source FOR VALUES IN (10);
CREATE TABLE detection_event_source_partition_month_id_11 PARTITION OF detection_event_source FOR VALUES IN (11);
CREATE TABLE detection_event_source_partition_month_id_12 PARTITION OF detection_event_source FOR VALUES IN (12);
CREATE TABLE detection_event_source_partition_month_id_13 PARTITION OF detection_event_source FOR VALUES IN (13);
CREATE TABLE detection_event_source_partition_month_id_14 PARTITION OF detection_event_source FOR VALUES IN (14);
CREATE TABLE detection_event_source_partition_month_id_15 PARTITION OF detection_event_source FOR VALUES IN (15);
CREATE TABLE detection_event_source_partition_month_id_16 PARTITION OF detection_event_source FOR VALUES IN (16);
CREATE TABLE detection_event_source_partition_month_id_17 PARTITION OF detection_event_source FOR VALUES IN (17);
CREATE TABLE detection_event_source_partition_month_id_18 PARTITION OF detection_event_source FOR VALUES IN (18);
CREATE TABLE detection_event_source_partition_month_id_19 PARTITION OF detection_event_source FOR VALUES IN (19);
CREATE TABLE detection_event_source_partition_month_id_20 PARTITION OF detection_event_source FOR VALUES IN (20);
CREATE TABLE detection_event_source_partition_month_id_21 PARTITION OF detection_event_source FOR VALUES IN (21);
CREATE TABLE detection_event_source_partition_month_id_22 PARTITION OF detection_event_source FOR VALUES IN (22);
CREATE TABLE detection_event_source_partition_month_id_23 PARTITION OF detection_event_source FOR VALUES IN (23);

CREATE UNIQUE INDEX detection_event_source_unique ON detection_event_source (
    detection_event_uuid,
    src_compute_uuid,
    src_compute_type,
    partition_month_id
);

CREATE INDEX detection_event_source_event_uuid ON detection_event_source (detection_event_uuid);

-- index on uuid before type, so in a pinch we can find a uuid without knowing type
CREATE INDEX detection_event_source_source ON detection_event_source (src_compute_uuid, src_compute_type, start_time);

--------------------------------------------------------------------
-- detection_event_flow ------------------------------------------

CREATE TABLE detection_event_flow(
    detection_event_flow_uuid UUID NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    detection_event_uuid UUID NOT NULL,
    observable_value_hash VARCHAR(32) NOT NULL,
    client_compute_type compute_type_enum NOT NULL,
    client_compute_uuid UUID NOT NULL,
    client_compute_ip INET NULL,
    client_compute_fqdn VARCHAR(255) NULL,
    client_is_private BOOL NOT NULL,
    client_port INTEGER NULL,
    client_site_uuid UUID NULL,
    server_compute_type compute_type_enum NOT NULL,
    server_compute_uuid UUID NOT NULL,
    server_compute_ip INET NULL,
    server_compute_fqdn VARCHAR(255) NULL,
    server_is_private BOOL NOT NULL,
    server_port INTEGER NULL,
    server_site_uuid UUID NULL,
    transport_protocol transport_protocol_enum NULL,
    application_protocol VARCHAR(255) NULL,
    transport_node_uuid UUID NULL,
    transport_node_type transport_node_type_enum NULL,
    verification_outcome verification_outcome_enum NOT NULL,
    verifier_uuid UUID NULL,
    pcap_uuid UUID NULL,
    client_username TEXT DEFAULT NULL,
    server_username TEXT DEFAULT NULL,
    PRIMARY KEY (detection_event_flow_uuid, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_flow_partition_month_id_0 PARTITION OF detection_event_flow FOR VALUES IN (0);
CREATE TABLE detection_event_flow_partition_month_id_1 PARTITION OF detection_event_flow FOR VALUES IN (1);
CREATE TABLE detection_event_flow_partition_month_id_2 PARTITION OF detection_event_flow FOR VALUES IN (2);
CREATE TABLE detection_event_flow_partition_month_id_3 PARTITION OF detection_event_flow FOR VALUES IN (3);
CREATE TABLE detection_event_flow_partition_month_id_4 PARTITION OF detection_event_flow FOR VALUES IN (4);
CREATE TABLE detection_event_flow_partition_month_id_5 PARTITION OF detection_event_flow FOR VALUES IN (5);
CREATE TABLE detection_event_flow_partition_month_id_6 PARTITION OF detection_event_flow FOR VALUES IN (6);
CREATE TABLE detection_event_flow_partition_month_id_7 PARTITION OF detection_event_flow FOR VALUES IN (7);
CREATE TABLE detection_event_flow_partition_month_id_8 PARTITION OF detection_event_flow FOR VALUES IN (8);
CREATE TABLE detection_event_flow_partition_month_id_9 PARTITION OF detection_event_flow FOR VALUES IN (9);
CREATE TABLE detection_event_flow_partition_month_id_10 PARTITION OF detection_event_flow FOR VALUES IN (10);
CREATE TABLE detection_event_flow_partition_month_id_11 PARTITION OF detection_event_flow FOR VALUES IN (11);
CREATE TABLE detection_event_flow_partition_month_id_12 PARTITION OF detection_event_flow FOR VALUES IN (12);
CREATE TABLE detection_event_flow_partition_month_id_13 PARTITION OF detection_event_flow FOR VALUES IN (13);
CREATE TABLE detection_event_flow_partition_month_id_14 PARTITION OF detection_event_flow FOR VALUES IN (14);
CREATE TABLE detection_event_flow_partition_month_id_15 PARTITION OF detection_event_flow FOR VALUES IN (15);
CREATE TABLE detection_event_flow_partition_month_id_16 PARTITION OF detection_event_flow FOR VALUES IN (16);
CREATE TABLE detection_event_flow_partition_month_id_17 PARTITION OF detection_event_flow FOR VALUES IN (17);
CREATE TABLE detection_event_flow_partition_month_id_18 PARTITION OF detection_event_flow FOR VALUES IN (18);
CREATE TABLE detection_event_flow_partition_month_id_19 PARTITION OF detection_event_flow FOR VALUES IN (19);
CREATE TABLE detection_event_flow_partition_month_id_20 PARTITION OF detection_event_flow FOR VALUES IN (20);
CREATE TABLE detection_event_flow_partition_month_id_21 PARTITION OF detection_event_flow FOR VALUES IN (21);
CREATE TABLE detection_event_flow_partition_month_id_22 PARTITION OF detection_event_flow FOR VALUES IN (22);
CREATE TABLE detection_event_flow_partition_month_id_23 PARTITION OF detection_event_flow FOR VALUES IN (23);

CREATE UNIQUE INDEX detection_event_flow_unique ON detection_event_flow (
    detection_event_uuid,
    observable_value_hash,
    partition_month_id
);

CREATE INDEX detection_event_flow_start_time ON detection_event_flow (start_time);

CREATE INDEX detection_event_flow_pcap ON detection_event_flow (pcap_uuid);

--------------------------------------------------------------------
-- campaign --------------------------------------------------------

CREATE TABLE campaign(
    campaign_uuid UUID PRIMARY KEY,
    -- Start time of campaign (start of first detection in campaign)
    start_time TIMESTAMP NOT NULL,
    -- End time of campaign (end of last detection in campaign)
    -- This column is used for data retention.
    end_time TIMESTAMP NOT NULL,
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    last_modified TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    -- Human-readable name of campaign
    campaign_name VARCHAR(255) NOT NULL,
    -- True if the campaign name was set by the user
    user_defined_campaign_name BOOLEAN NOT NULL DEFAULT False,
    -- "last" here means most advanced, as ordered within mitre framework: Not chronologically last
    last_mitre_tactic_uuid UUID NOT NULL,
    impact SMALLINT NOT NULL,
    num_affected_workloads INTEGER NOT NULL,
    num_threats INTEGER NOT NULL,
    num_detection_events INTEGER NOT NULL,
    num_sites INTEGER NOT NULL
);

-- index on end_time since that is the column used for data retention
CREATE INDEX campaign_start_time ON campaign (end_time);

CREATE TRIGGER update_campaign_modtime
    BEFORE UPDATE ON campaign
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_last_modified_column();

--------------------------------------------------------------------
-- detection_event_campaign ----------------------------------------

CREATE TABLE detection_event_campaign(
    -- unique identifier of detection event (logical foreign key)
    detection_event_uuid UUID NOT NULL,
    start_time TIMESTAMP NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    -- unique identifier of campaign (logical foreign key)
    campaign_uuid UUID NOT NULL,
    PRIMARY KEY (detection_event_uuid, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detection_event_campaign_partition_month_id_0 PARTITION OF detection_event_campaign FOR VALUES IN (0);
CREATE TABLE detection_event_campaign_partition_month_id_1 PARTITION OF detection_event_campaign FOR VALUES IN (1);
CREATE TABLE detection_event_campaign_partition_month_id_2 PARTITION OF detection_event_campaign FOR VALUES IN (2);
CREATE TABLE detection_event_campaign_partition_month_id_3 PARTITION OF detection_event_campaign FOR VALUES IN (3);
CREATE TABLE detection_event_campaign_partition_month_id_4 PARTITION OF detection_event_campaign FOR VALUES IN (4);
CREATE TABLE detection_event_campaign_partition_month_id_5 PARTITION OF detection_event_campaign FOR VALUES IN (5);
CREATE TABLE detection_event_campaign_partition_month_id_6 PARTITION OF detection_event_campaign FOR VALUES IN (6);
CREATE TABLE detection_event_campaign_partition_month_id_7 PARTITION OF detection_event_campaign FOR VALUES IN (7);
CREATE TABLE detection_event_campaign_partition_month_id_8 PARTITION OF detection_event_campaign FOR VALUES IN (8);
CREATE TABLE detection_event_campaign_partition_month_id_9 PARTITION OF detection_event_campaign FOR VALUES IN (9);
CREATE TABLE detection_event_campaign_partition_month_id_10 PARTITION OF detection_event_campaign FOR VALUES IN (10);
CREATE TABLE detection_event_campaign_partition_month_id_11 PARTITION OF detection_event_campaign FOR VALUES IN (11);
CREATE TABLE detection_event_campaign_partition_month_id_12 PARTITION OF detection_event_campaign FOR VALUES IN (12);
CREATE TABLE detection_event_campaign_partition_month_id_13 PARTITION OF detection_event_campaign FOR VALUES IN (13);
CREATE TABLE detection_event_campaign_partition_month_id_14 PARTITION OF detection_event_campaign FOR VALUES IN (14);
CREATE TABLE detection_event_campaign_partition_month_id_15 PARTITION OF detection_event_campaign FOR VALUES IN (15);
CREATE TABLE detection_event_campaign_partition_month_id_16 PARTITION OF detection_event_campaign FOR VALUES IN (16);
CREATE TABLE detection_event_campaign_partition_month_id_17 PARTITION OF detection_event_campaign FOR VALUES IN (17);
CREATE TABLE detection_event_campaign_partition_month_id_18 PARTITION OF detection_event_campaign FOR VALUES IN (18);
CREATE TABLE detection_event_campaign_partition_month_id_19 PARTITION OF detection_event_campaign FOR VALUES IN (19);
CREATE TABLE detection_event_campaign_partition_month_id_20 PARTITION OF detection_event_campaign FOR VALUES IN (20);
CREATE TABLE detection_event_campaign_partition_month_id_21 PARTITION OF detection_event_campaign FOR VALUES IN (21);
CREATE TABLE detection_event_campaign_partition_month_id_22 PARTITION OF detection_event_campaign FOR VALUES IN (22);
CREATE TABLE detection_event_campaign_partition_month_id_23 PARTITION OF detection_event_campaign FOR VALUES IN (23);

CREATE INDEX detection_event_campaign_start_time ON detection_event_campaign (start_time);
CREATE INDEX detection_event_campaign_campaign_uuid ON detection_event_campaign (campaign_uuid, start_time);

--------------------------------------------------------------------
-- campaign_graph --------------------------------------------------

CREATE TABLE campaign_graph(
    campaign_uuid UUID PRIMARY KEY,
    -- Needed in case we later make backwards-incompatible changes to the JSON format of campaign graphs
    campaign_graph_format_version SMALLINT NOT NULL DEFAULT (1),
    -- Json representation of campaign graph
    campaign_graph JSONB NOT NULL,
    CONSTRAINT fk_campaign FOREIGN KEY(campaign_uuid) REFERENCES campaign(campaign_uuid)
    ON DELETE CASCADE
);

--------------------------------------------------------------------
-- campaign_merge_log ----------------------------------------------

CREATE TABLE campaign_merge_log(
    campaign_merge_log_uuid UUID PRIMARY KEY,
    -- Uuid of the campaign that no longer exists, because it was merged into another one
    source_campaign_uuid UUID NOT NULL,
    -- Uuid of the target campaign that the source campaign was merged into.
    target_campaign_uuid UUID NOT NULL,
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    -- This is the column used for data retention.
    last_modified  TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc')
);

CREATE UNIQUE INDEX campaign_merge_log_unique_source_campaign_uuid ON campaign_merge_log (source_campaign_uuid);
CREATE INDEX campaign_merge_log_target_campaign_uuid ON campaign_merge_log (target_campaign_uuid);

CREATE TRIGGER update_campaign_merge_log_modtime
    BEFORE UPDATE ON campaign_merge_log
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_last_modified_column();

--------------------------------------------------------------------
-- campaign_mitre_tactic -------------------------------------------

CREATE TABLE campaign_mitre_tactic(
    campaign_uuid UUID NOT NULL,
    mitre_tactic_uuid UUID NOT NULL,
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    PRIMARY KEY (campaign_uuid, mitre_tactic_uuid),
    CONSTRAINT fk_campaign FOREIGN KEY(campaign_uuid) REFERENCES campaign(campaign_uuid)
    ON DELETE CASCADE
);

CREATE INDEX campaign_mitre_tactic_mitre_tactic ON campaign_mitre_tactic (mitre_tactic_uuid);

--------------------------------------------------------------------
-- siem_notification -----------------------------------------------

CREATE TABLE siem_notification(
    siem_notification_uuid UUID PRIMARY KEY,
    -- URL to post notifications to
    url TEXT NOT NULL,
    -- HTTP headers to include in POST
    headers JSONB NULL DEFAULT NULL,
    disable_ssl_verification BOOLEAN NOT NULL DEFAULT FALSE,
    message_format siem_message_format_enum NOT NULL,
    enabled BOOLEAN NOT NULL DEFAULT TRUE,
    -- User-friendly name of the configuration, for display in UI
    display_name VARCHAR(255) NOT NULL,
    -- Longer-form description of this configuration
    description VARCHAR(1024) NULL DEFAULT NULL,
    insert_time TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    last_modified  TIMESTAMP NOT NULL DEFAULT (now() at time zone 'utc'),
    -- last verification details
    last_verification_time TIMESTAMP DEFAULT NULL,
    last_verification_successful BOOLEAN DEFAULT NULL,
    last_verification_message VARCHAR(1024) DEFAULT NULL,
    -- Generation/revision of this configuration (incremented by one on each update)
    revision BIGINT NOT NULL DEFAULT 1,
    -- optional frontend fqdn for links to the NDR UI
    frontend_fqdn VARCHAR(255) DEFAULT NULL
);


CREATE TRIGGER update_siem_notification_modtime
    BEFORE UPDATE ON siem_notification
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_last_modified_column();

CREATE TRIGGER update_siem_notification_revision
    BEFORE UPDATE ON siem_notification
    FOR EACH ROW
    WHEN (OLD IS DISTINCT FROM NEW)
    EXECUTE PROCEDURE update_revision_column();


--------------------------------------------------------------------
-- file detections -------------------------------------------------

CREATE TYPE file_verdict_enum AS ENUM ('BENIGN', 'SUSPICIOUS', 'MALICIOUS', 'ERROR');

CREATE TYPE file_category_enum AS ENUM ('EXECUTABLE', 'DOCUMENT', 'SCRIPT', 'ARCHIVE', 'DATA', 'MEDIA', 'OTHER');

CREATE TABLE detected_file(
    -- sha256 stored in hex for join with detection_event.indicator_of_compromise
    file_sha256 VARCHAR(64) NOT NULL,
    partition_month_id SMALLINT NOT NULL,
    start_time TIMESTAMP NOT NULL,
    end_time TIMESTAMP NOT NULL,
    file_md5 UUID NOT NULL,
    -- sha1 stored in hex
    file_sha1 VARCHAR(40) NOT NULL,
    analysis_task_uuid UUID NULL,
    score SMALLINT NOT NULL,
    verdict file_verdict_enum NOT NULL,
    file_size INT NULL,
    lastline_file_class VARCHAR(64) NOT NULL,
    file_category file_category_enum NOT NULL,
    PRIMARY KEY (file_sha256, partition_month_id)
) PARTITION BY LIST (partition_month_id);

CREATE TABLE detected_file_partition_month_id_0 PARTITION OF detected_file FOR VALUES IN (0);
CREATE TABLE detected_file_partition_month_id_1 PARTITION OF detected_file FOR VALUES IN (1);
CREATE TABLE detected_file_partition_month_id_2 PARTITION OF detected_file FOR VALUES IN (2);
CREATE TABLE detected_file_partition_month_id_3 PARTITION OF detected_file FOR VALUES IN (3);
CREATE TABLE detected_file_partition_month_id_4 PARTITION OF detected_file FOR VALUES IN (4);
CREATE TABLE detected_file_partition_month_id_5 PARTITION OF detected_file FOR VALUES IN (5);
CREATE TABLE detected_file_partition_month_id_6 PARTITION OF detected_file FOR VALUES IN (6);
CREATE TABLE detected_file_partition_month_id_7 PARTITION OF detected_file FOR VALUES IN (7);
CREATE TABLE detected_file_partition_month_id_8 PARTITION OF detected_file FOR VALUES IN (8);
CREATE TABLE detected_file_partition_month_id_9 PARTITION OF detected_file FOR VALUES IN (9);
CREATE TABLE detected_file_partition_month_id_10 PARTITION OF detected_file FOR VALUES IN (10);
CREATE TABLE detected_file_partition_month_id_11 PARTITION OF detected_file FOR VALUES IN (11);
CREATE TABLE detected_file_partition_month_id_12 PARTITION OF detected_file FOR VALUES IN (12);
CREATE TABLE detected_file_partition_month_id_13 PARTITION OF detected_file FOR VALUES IN (13);
CREATE TABLE detected_file_partition_month_id_14 PARTITION OF detected_file FOR VALUES IN (14);
CREATE TABLE detected_file_partition_month_id_15 PARTITION OF detected_file FOR VALUES IN (15);
CREATE TABLE detected_file_partition_month_id_16 PARTITION OF detected_file FOR VALUES IN (16);
CREATE TABLE detected_file_partition_month_id_17 PARTITION OF detected_file FOR VALUES IN (17);
CREATE TABLE detected_file_partition_month_id_18 PARTITION OF detected_file FOR VALUES IN (18);
CREATE TABLE detected_file_partition_month_id_19 PARTITION OF detected_file FOR VALUES IN (19);
CREATE TABLE detected_file_partition_month_id_20 PARTITION OF detected_file FOR VALUES IN (20);
CREATE TABLE detected_file_partition_month_id_21 PARTITION OF detected_file FOR VALUES IN (21);
CREATE TABLE detected_file_partition_month_id_22 PARTITION OF detected_file FOR VALUES IN (22);
CREATE TABLE detected_file_partition_month_id_23 PARTITION OF detected_file FOR VALUES IN (23);

CREATE INDEX detected_file_md5 ON detected_file (file_md5);
CREATE INDEX detected_file_sha1 ON detected_file (file_sha1);
CREATE INDEX detected_file_analysis_task ON detected_file (analysis_task_uuid);
